const ControllerBase = {
  async create(request, response) {
    try {
      const doc = new this.model(request.body)

      const saved = await doc.save()

      return response.status(201).json(saved)
    } catch (error) {
      return response.status(this.getErrorStatus(error)).send({ message: 'Server error', error })
    }
  },

  async find(request, response) {
    try {
      const result = {
        [this.model.collection.collectionName.toLowerCase()]: await this.model.find(),
        metadata: { total: await this.model.count() },
      }

      return response.status(200).json(result)
    } catch (error) {
      return response.status(500).send({ message: 'Server error', error })
    }
  },

  async findById(request, response) {
    try {
      const doc = await this.model.findById(request.params.id)
      return response.status(200).json(doc)
    } catch (error) {
      return response.status(500).send({ message: 'Server error', error })
    }
  },

  async update(request, response) {
    try {
      const query = { _id: request.params.id }
      const update = request.body
      const doc = await this.model.findOneAndUpdate(query, update, { new: true })

      if (!doc) {
        return response.status(404).json({ message: 'Not found' })
      }

      return response.status(200).json(doc)
    } catch (error) {
      return response.status(500).send({ message: 'Server error', error })
    }
  },

  async remove(request, response) {
    try {
      const doc = await this.model.findOneAndRemove({ _id: request.params.id })

      if (!doc) {
        return response.status(404).json({ message: 'Not found' })
      }

      return response.status(200).json({ [this.model.modelName.toLowerCase()]: doc._id })
    } catch (error) {
      return response.status(500).send({ message: 'Server error', error })
    }
  },

  getErrorStatus(error) {
    return error.name === 'ValidationError' ? 400 : 500
  },
}

module.exports = { ControllerBase }
