const { ControllerBase } = require('./controller-base')
const { Controller } = require('./controller')

class ControllerFactory {
  constructor() {
    throw new Error('ControllerFactory includes only static methods and does not need to be instantiated') // eslint-disable-line
  }

  static getControllerFor(model, overrides = {}, base = ControllerBase) {
    if (!model) {
      throw new Error(`Can't instantiate controller for model '${model}'`)
    }

    const controller = new Controller(model)
    this.setControllerInterface(controller, base, overrides)
    this.autoBind(controller)

    return controller
  }

  static setControllerInterface(controller, base, overrides) {
    if (!(controller instanceof Controller)) {
      throw new Error('Object provided is not a Controller instance')
    }
    const proto = Object.getPrototypeOf(controller)

    const mixin = { ...base, ...overrides }

    Object.keys(mixin).forEach(method => {
      proto[method] = mixin[method]
    })
  }

  static autoBind(controller) {
    const proto = Object.getPrototypeOf(controller)
    const propertyNames = Object.getOwnPropertyNames(proto)

    propertyNames.forEach(name => {
      const propertyValue = proto[name]

      if (name === 'constructor') {
        return
      }

      if (typeof propertyValue === 'function') {
        proto[name] = proto[name].bind(controller)
      }
    })
  }
}

module.exports = ControllerFactory
